﻿/*globals document DOM EXEMPLES GUI confirm Arbre */

var Code = {
	debug: false
};

var TikZ = {
	Debut: "%:-+-+-+- Engendré par : http://math.et.info.free.fr/TikZ/Arbre/\n" +
		"\\begin{center}\n",
	Styles: "\\begin{tikzpicture}[xscale=1,yscale=1]\n" +
		"% Styles (MODIFIABLES)\n" +
		"\\tikzstyle{fleche}=[->,>=latex,thick]\n" +
		"\\tikzstyle{noeud}=[fill=yellow,circle,draw]\n" +
		"\\tikzstyle{feuille}=[fill=yellow,circle,draw]\n",
	StylesListe: "\\begin{tikzpicture}[xscale=1,yscale=1]\n" +
		"% Styles (MODIFIABLES)\n" +
		"\\tikzstyle{fleche}=[->,>=latex,thick,rounded corners=2mm]\n" +
		"\\tikzstyle{noeud}=[fill=yellow,draw]\n" +
		"\\tikzstyle{feuille}=[fill=yellow,draw]\n",
	Dimensions: "% Dimensions (MODIFIABLES)\n" +
		"\\def\\DistanceInterNiveaux{3}\n" +
		"\\def\\DistanceInterFeuilles{2}\n",
	DimensionsListe: "% Dimensions (MODIFIABLES)\n" +
		"\\def\\DistanceInterNiveaux{2}\n" +
		"\\def\\DistanceInterFeuilles{1}\n",
	SourceFin: "\\end{tikzpicture}\n" +
		"\\end{center}\n" +
		"%:-+-+-+-+- Fin\n"
};

//================= Compilation =================
//============= TikZ.creerTexteSource =============
TikZ.creerTexteSource = function () {
	Code.racine.rafraichir();
	TikZ.texteSource = TikZ.Debut;
	if (GUI.orientation === "Liste") {
		TikZ.texteSource += "% Racine en Haut à Gauche, développement en liste hiérarchisée\n";
	} else if (GUI.orientation === "Droite") {
		TikZ.texteSource += "% Racine à Droite, développement vers la gauche\n";
	} else if (GUI.orientation === "Haut") {
		TikZ.texteSource += "% Racine en Haut, développement vers le bas\n";
	} else if (GUI.orientation === "Bas") {
		TikZ.texteSource += "% Racine en Bas, développement vers le haut\n";
	} else {
		TikZ.texteSource += "% Racine à Gauche, développement vers la droite\n";
	}
	if (GUI.orientation === "Liste") {
		TikZ.texteSource += TikZ.StylesListe;
	} else {
		TikZ.texteSource += TikZ.Styles;
	}
	// Étiquetes d'arcs éventuelles
	if (GUI.avecArcs) {
		if (GUI.orientation === "Liste") {
			TikZ.texteSource += "\\tikzstyle{etiquette}=[near end,above]\n";
		} else {
			TikZ.texteSource += "\\tikzstyle{etiquette}=[midway,fill=white,draw]\n";
		}
	}
	// Commentaires éventuels
	if (GUI.avecCommentaires) {
		if (GUI.orientation === "Droite") {
			TikZ.texteSource += "\\tikzstyle{commentaire}=[left,blue]\n";
		} else if (GUI.orientation === "Haut") {
			TikZ.texteSource += "\\tikzstyle{commentaire}=[below,blue]\n";
		} else if (GUI.orientation === "Bas") {
			TikZ.texteSource += "\\tikzstyle{commentaire}=[above,blue]\n";
		} else {
			TikZ.texteSource += "\\tikzstyle{commentaire}=[right,blue]\n";
		}
	}
	// Dimensions
	if (GUI.orientation === "Liste") {
		TikZ.texteSource += TikZ.DimensionsListe;
	} else {
		TikZ.texteSource += TikZ.Dimensions;
	}
	// Dimensions calculées
	TikZ.texteSource += Code.racine.dimensionsTikZ();
	// code source des Noeuds de l'arbre
	TikZ.texteSource += "% Noeuds (MODIFIABLES : Styles et Coefficients d'InterFeuilles)\n";
	TikZ.texteSource += Code.racine.noeudsTikZ();
	// code source des Arcs de l'arbre 
	TikZ.texteSource += "% Arcs (MODIFIABLES : Styles)\n";
	TikZ.texteSource += Code.racine.arcsTikZ();
	TikZ.texteSource += TikZ.SourceFin;
	Code.afficherReponse(TikZ.texteSource);
};

//=================== Services ===================
//============= Code.afficherReponse =============
Code.afficherReponse = function (txt) {
	GUI.dom.texte.value = txt;
	GUI.dom.texte.focus();
	GUI.dom.texte.select();
};

//================== Contrôle Interface ==================
//================== Code.selectExo ==================
Code.selectExo = function () {
	if (GUI.dom.menu.selectedIndex === 0) {
		return;
	}
	Code.numeroExo = GUI.dom.menu.selectedIndex - 1;
	Code.afficherReponse("Exerice n° " + Code.numeroExo);
	Code.racine.init();
	Code.racine.definrAvec(EXEMPLES[Code.numeroExo]);
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
	GUI.dom.menu.selectedIndex = 0;
};

//================== Code.clicRAZ ==================
Code.clicRAZ = function () {
	Code.racine.visualiser(true);
	if (confirm("Voulez-vous vraiment effacer totalement cet arbre ?")) {
		Code.racine.visualiser(false);
		Code.afficherReponse("");
		DOM.effacerElement(GUI.dom.jeu);
		Code.racine.init();
		Code.racine.creerWidget(GUI.dom.jeu);
	}
	Code.racine.visualiser(false);
};

//================== Code.clicValider ==================
Code.clicValider = function () {
	TikZ.creerTexteSource();
};

//================== Code.clicDebug ==================
Code.clicDebug = function () {
	Code.debug = !Code.debug;
	Code.numeroExo = undefined;
	if (Code.debug) {
		GUI.dom.menuDebug.className = null;
	} else {
		GUI.dom.menuDebug.className = "invisible";
	}
};

//================== Code.defExo ==================
Code.defExo = function () {
	var txt;
	Code.racine.rafraichir();
	txt = Code.racine.prettyPrint();
	txt = txt.replace(/\\/g, "\\\\");
	Code.afficherReponse(txt);		
};

//================== Code.chargeExo ==================
Code.chargeExo = function () {
	var def;
	try {
		eval("def = " + GUI.dom.texte.value + ";");
	} catch (err) {
		Code.afficherReponse(err);
		return;
	}
	try {
		Code.racine.init();
		Code.racine.definrAvec(def);
		DOM.effacerElement(GUI.dom.jeu);
		Code.racine.creerWidget(GUI.dom.jeu);
	} catch (expt) {
		Code.afficherReponse(expt);
	}
};

//============= Code.clicAjouterFils =============
Code.clicAjouterFils = function (widget) {
	Code.racine.rafraichir();
	widget.arbre.ajouterFils();
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
};

//============= Code.clicMonter =============
Code.clicMonter = function (widget) {
	Code.racine.rafraichir();
	widget.arbre.monterFils();
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
};

//============= Code.clicDescendre =============
Code.clicDescendre = function (widget) {
	Code.racine.rafraichir();
	widget.arbre.descendreFils();
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
};

//============= Code.clicTuer =============
Code.clicTuer = function (widget) {
	widget.arbre.visualiser(true);
	if (confirm("Voulez-vous vraiment supprimer cet arbre ?")) {
		widget.arbre.visualiser(false);
		Code.racine.rafraichir();
		widget.arbre.tuer();
		DOM.effacerElement(GUI.dom.jeu);
		Code.racine.creerWidget(GUI.dom.jeu);
	}
	widget.arbre.visualiser(false);
};

//============= Code.clicArcs =============
Code.clicArcs = function (widget) {
	GUI.avecArcs = widget.checked;
	Code.racine.rafraichir();
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
	TikZ.creerTexteSource();
};

//================== Code.clicCommentaires ==================
Code.clicCommentaires = function (widget) {
	GUI.avecCommentaires = widget.checked;
	Code.racine.rafraichir();
	DOM.effacerElement(GUI.dom.jeu);
	Code.racine.creerWidget(GUI.dom.jeu);
	TikZ.creerTexteSource();
};

//============= GUI.clicOrientation =============
Code.clicOrientation = function (widget) {
	GUI.orientation = widget.value;
	TikZ.creerTexteSource();
};

//============= GUI.clicNiveaux =============
Code.clicNiveaux = function (widget) {
	GUI.niveauxEquidistants = widget.checked;
	TikZ.creerTexteSource();
};

//============= Code.clicModeEdit =============
Code.clicModeEdit = function (widget) {
	GUI.clicModeEdit(widget);
	TikZ.creerTexteSource();
};

//=========== Initialisation du Code ============
//================== Code.init ==================
Code.init = function () {
	Code.debug = false;
	GUI.creerMenuDebug(EXEMPLES.length);
	Code.racine = new Arbre();
	Code.racine.creerWidget(GUI.dom.jeu);
};
